<?php
/**
 * $Horde: klutz/backend.php,v 1.54 2007/01/02 00:41:33 jan Exp $
 *
 * Copyright 2002-2006 Marcus I. Ryan <marcus@riboflavin.net>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

$compression_control = 'none';
@define('AUTH_HANDLER', true);
@define('KLUTZ_BASE', dirname(__FILE__));
require_once KLUTZ_BASE . '/lib/base.php';
require_once 'Horde/CLI.php';

// Check for a Horde administrator.
if (!Auth::isAdmin()) {
    // Check for HTTP auth.
    if (isset($conf['auth']['http']) && is_array($conf['auth']['http'])) {
        if (empty($_SERVER['PHP_AUTH_USER']) ||
            ($_SERVER['PHP_AUTH_USER'] !== $conf['auth']['http']['username']) &&
            empty($_SERVER['PHP_AUTH_PW']) ||
            empty($conf['auth']['http']['password']) ||
            ($_SERVER['PHP_AUTH_PW'] !== $conf['auth']['http']['password']) ||
            !in_array($_SERVER['REMOTE_ADDR'], $conf['auth']['http']['ips'])) {

            header('WWW-Authenticate: Basic realm="Klutz Administration"');
            header('HTTP/1.0 401 Unauthorized');
            exit('forbidden');
        }
    } else {
        exit('forbidden');
    }
}

// Get a Horde_CLI instance.
$cli = &Horde_CLI::singleton();

// Don't do anything if the backend is none.
if ($conf['storage']['driver'] == 'none') {
    if ($cli->runningFromCLI()) {
        fwrite(STDERR, _("You must define a storage backend to use the administrative interface."));
        exit(1);
    } else {
        $notification->push(_("You must define a storage backend to use the administrative interface."), 'horde.error');
        header('Location: ' . Horde::applicationUrl('comics.php'));
        exit;
    }
}

// No more notifications after this point, so close the session so we
// don't lock other pages.
session_write_close();

// Proceed once we're sure we're authorized.
if (empty($mode)) {
    $mode = Util::getFormData('mode', array('menu'));
}
if (!is_array($mode)) {
    $mode = array($mode);
}

$time = mktime(0, 0, 0);
$t = getdate($time);
if (!empty($conf['backend']['daystokeep'])) {
    $daystokeep = $conf['backend']['daystokeep'];
    $oldest = mktime(0, 0, 0, $t['mon'], $t['mday'] - $daystokeep, $t['year']);
} else {
    $daystokeep = 90;
    $oldest = null;
}
$expired = mktime(0, 0, 0, $t['mon'], $t['mday'] - ($daystokeep + 1), $t['year']);

if (in_array('sums', $mode)) {
    $storage->rebuildSums();
    if (!in_array('menu', $mode)) {
        echo _("The unique identifiers table has been rebuilt.");
    }
}

if (count($mode) == 0 || in_array('menu', $mode)) {
    $fetch_date_select = '<select name="date">' .
        '<option value="all">' . _("All Dates") . '</option>';
    foreach (array_reverse(Klutz_Driver::listDates($time, $oldest)) as $date) {
        if ($date == $time) {
            $selected = ' selected="selected"';
        } else {
            $selected = '';
        }
        $fetch_date_select .= '<option value="' . $date . '"' . $selected . '">' .
            strftime('%B %d, %Y', $date) . '</option>';
    }
    $fetch_date_select .= '</select>';

    $delete_date_select = '<select name="date">';
    foreach ($storage->listDates($time, 0, $time) as $date) {
        if ($date == $oldest) {
            $selected = ' selected="selected"';
        } else {
            $selected = '';
        }
        $delete_date_select .= '<option value="' . $date . '"' . $selected . '">' .
            strftime("%B %d, %Y", $date) . '</option>';
    }
    $delete_date_select .= '</select>';

    $comic_select = '<select name="index"><option value="all" selected="selected">' .
        _("All Comics") . '</option>';
    foreach ($klutz->listEnabled() as $index) {
        $comic_select .= "<option value=\"$index\">" .
            $klutz->getProperty($index, 'name') .
            '</option>';
    }
    $comic_select .= '</select>';

    $sums_url = Horde::applicationUrl('backend.php');
    $sums_url = Util::addParameter($sums_url, 'mode[]', 'sums');
    $sums_url = Util::addParameter($sums_url, 'mode[]', 'menu');

    $title = _("Comics Update");
    require KLUTZ_TEMPLATES . '/common-header.inc';
    require KLUTZ_TEMPLATES . '/menu.inc';
    require KLUTZ_TEMPLATES . '/backend/backend.inc';
    require $registry->get('templates', 'horde') . '/common-footer.inc';
    exit;
}

if (in_array('fetch', $mode)) {
    if (empty($date)) {
        $date = Util::getFormData('date', mktime(0, 0, 0));
    }
    if (empty($index)) {
        $index = Util::getFormData('index', 'all');
    }
    if (empty($overwrite)) {
        $overwrite = Util::getFormData('overwrite', 'false');
    }
    if (empty($nounique)) {
        $nounique = Util::getFormData('nounique', 'false');
    }

    if (String::lower($overwrite) == 'true') {
        $overwrite = true;
    } else {
        $overwrite = false;
    }
    if (String::lower($nounique) == 'true') {
        $nounique = true;
    } else {
        $nounique = false;
    }

    if (String::lower($date) == 'all') {
        $dates = array_reverse(Klutz_Driver::listDates($time, $oldest));
    } else {
        // make sure the time for $date is midnight
        $d = getdate($date);
        $date = mktime(0, 0, 0, $d['mon'], $d['mday'], $d['year']);
        $dates = array($date);
    }

    foreach ($dates as $date) {
        // is $date today?
        $today = false;
        if ($date != 'all' && $date == mktime(0, 0, 0)) {
            $today = true;
        }

        if (String::lower($index) == 'all') {
            $comics = $klutz->listEnabled(null, $date);
        } else {
            $comics = $klutz->listEnabled(array($index), $date);
        }

        foreach ($comics as $comic) {
            // If this comic isn't available all days, nohistory gets
            // ugly. SOMEONE PLEASE MAKE IT BETTER ;)
            $name = $klutz->getProperty($comic, 'name');
            $days = $klutz->getProperty($comic, 'days');
            if ($days != 'random' && count($days) < 7) {
                $day = mktime(0, 0, 0);
                while (!in_array(String::lower(date('D', $day)),$days)) {
                    $d = getdate($day);
                    $day = mktime(0, 0, 0, $d['mon'], $d['mday'] - 1, $d['year']);
                }
                if ($day == $date) {
                    $today = true;
                }
            }

            if (!$today && $klutz->getProperty($comic, 'nohistory')) {
                webPrint(sprintf(_("Skipping %s for %s - no historical fetching"),
                                 $name, strftime("%B %d, %Y", $date)));
            } elseif (!$storage->imageExists($comic, $date) || $overwrite) {
                webPrint(sprintf(_("Fetching %s for date %s..."),
                                 $name, strftime('%B %d, %Y', $date)));
                flush();

                $c = $klutz->comicObject($comic);
                $image = $c->fetchImage($date);
                if (!is_object($image)) {
                    webPrint(_("The image doesn't exist or can't be retrieved"));
                } elseif (!$nounique && (method_exists($storage, "isUnique") &&
                          !$storage->isUnique($image))) {
                    webPrint(sprintf(_("This image appears to be a repeat (%s)"), md5($image->data)));
                } elseif ($storage->storeImage($comic, $image, $date)) {
                    webPrint(_("Done"));
                } else {
                    webPrint(_("Error storing the image"));
                }
            } else {
                webPrint(sprintf(_("Skipping %s for %s"),
                                 $name, strftime('%B %d, %Y', $date)));
            }
            webPrint("<br />\n");
        }
    }

    // Save the updated sums if necessary
    if (method_exists($storage, 'saveSums')) {
        $storage->saveSums();
    }
}

if (in_array('delete', $mode)) {
    // Note: We don't allow overriding of $date from backend.php because it
    // would conflict with the $date used for fetching so we'd delete all
    // comics if fetch and delete were both requested in one file.
    $date = Util::getFormData('date', $expired);
    $index = Util::getFormData('index', 'all');
    $timeframe = Util::getFormData('timeframe', 'older');

    // make sure the time for $date is midnight
    $d = getdate($date);
    $date = mktime(0, 0, 0, $d['mon'], $d['mday'], $d['year']);

    $success = _("Successfully removed %s for %s<br />\n");
    $notfound = _("Could not find %s for %s<br />\n");
    $error = _("An error occurred removing %s for %s<br />\n");

    // Figure out what dates we need to work on
    if ($timeframe == 'date') {
        $dates = array($date);
    } elseif ($timeframe == 'older') {
        $dates = $storage->listDates($date, 0, $date);
    } elseif ($timeframe == 'newer') {
        $dates = $storage->listDates($date, $date, mktime(0, 0, 0));
    } else {
        $dates = array();
    }

    if ($index == 'all') {
        if (count($dates) == 0) {
            webPrint(sprintf($notfound, _("any comics"), _("any dates")));
        }
        foreach ($dates as $date) {
            if ($storage->removeDate($date)) {
                webPrint(sprintf($success, _("all comics"), strftime('%B %d, %Y', $date)));
            } else {
                webPrint(sprintf($error, _("all comics"), strftime('%B %d, %Y', $date)));
            }
        }
    } else {
        $name = $klutz->getProperty($index, 'name');
        $days = $klutz->getProperty($index, 'days');

        foreach ($dates as $date) {
            if ($days == 'random' || in_array(String::lower(date('D', $date)), $days)) {
                if (!$storage->imageExists($index, $date)) {
                    webPrint(sprintf($notfound, $index, strftime('%B %d, %Y', $date)));
                } else {
                    if ($storage->removeImage($index, $date)) {
                        webPrint(sprintf($success, $name, strftime('%B %d, %Y', $date)));
                    } else {
                        webPrint(sprintf($error, $name, strftime('%B %d, %Y', $date)));
                    }
                }
            }
        }
    }

    // Save the updated sums if necessary.
    if (method_exists($storage, 'saveSums')) {
        $storage->saveSums();
    }
}


/**
 * Output text only if this is the web interface.
 *
 * @param string $text  The text to print.
 */
function webPrint($text)
{
    global $cli;

    if (!$cli->runningFromCLI()) {
        echo $text;
    }
}
